import { dbconn } from 'decorators/dbconn'
import { getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { BaseModule } from './Base'

@modelOptions({ schemaOptions: { _id: false } })
export class Item {
  @prop()
  id: string
  @prop()
  type: number
  @prop()
  name: string
  @prop({ default: 1 })
  amount: number
}
/**
 * 用户大转盘抽奖记录
 */
@dbconn()
@index({ user: 1 }, { unique: false })
@index({ user: 1, dateTag: 1 }, { unique: false })
@modelOptions({
  schemaOptions: { collection: 'draw_record', timestamps: true },
})
class DrawRecordClass extends BaseModule {
  @prop()
  public user: string
  @prop()
  public dateTag: string
  @prop()
  public position: number
  // 获得的游戏内积分数量
  @prop({ default: 0 })
  public score: number

  @prop({ type: () => [Item], default: [] })
  public items: Item[]

  public toJson() {
    return {
      day: this.dateTag,
      score: this.score,
      items: this.items,
    }
  }
}
export const DrawRecord = getModelForClass(DrawRecordClass, { existingConnection: DrawRecordClass['db'] })
