import { Severity, getModelForClass, index, modelOptions, prop } from '@typegoose/typegoose'
import { dbconn } from 'decorators/dbconn'
import { BaseModule } from './Base'

/**
 * ingame task record
 */

@dbconn()
@index({ user: 1, dateTag: 1 }, { unique: false })
@index({ user: 1, task: 1 }, { unique: false })
@index({ user: 1, task: 1, dateTag: 1 }, { unique: true })
@modelOptions({
  schemaOptions: { collection: 'ingame_task_record', timestamps: true },
  options: { allowMixed: Severity.ALLOW },
})
class InGameTaskRecordClass extends BaseModule {
  @prop({ required: true })
  public user: string
  @prop({ required: true })
  public task: string
  @prop()
  public dateTag: string
  // 获得的积分数量
  @prop({ default: 0 })
  public score: number
  // 获得的抽奖次数
  @prop({ default: 0 })
  public ticket: number
}

export const InGameTaskRecord = getModelForClass(InGameTaskRecordClass, {
  existingConnection: InGameTaskRecordClass['db'],
})
