import { RECAPTCHA_MIN_SCORE } from 'common/Constants'
import { OAuth2Client } from 'google-auth-library'
import logger from 'logger/logger'
import { ZError } from 'zutils'

export const checkReCaptcha = async (req: any, action: string) => {
  if (!(process.env.NEED_RECAPTCHA && process.env.NEED_RECAPTCHA === '1')) {
    return
  }
  const { rtoken } = req.params
  if (!rtoken) {
    throw new ZError(50, 'reCaptcha token is required')
  }
  const url = `https://www.google.com/recaptcha/api/siteverify?secret=${process.env.RECAPTCHA_SECRET}&response=${rtoken}`
  const response = await fetch(url, {
    method: 'POST',
  })
  const data = await response.json()
  if (!data.success) {
    throw new ZError(51, `reCaptcha invalid`)
  }
  if (data.action !== action) {
    throw new ZError(52, `reCaptcha action invalid`)
  }
  if (data.score < RECAPTCHA_MIN_SCORE) {
    throw new ZError(53, `reCaptcha score invalid`)
  }
  return true
}

const GOOGLE_OAUTH_ISS = 'https://accounts.google.com'
const GOOGLE_OAUTH_ISS1 = 'accounts.google.com'
const IOS_TEST = '53206975661-0d6q9pqljn84n9l63gm0to1ulap9cbk4.apps.googleusercontent.com'
const CLIENT_ID = '53206975661-asnf3qe4bg29p8h981pgf099osvrjbme.apps.googleusercontent.com'
const CLIENT_ID2 = '53206975661-ih3r0ubph3rqejdq97b029difbrk2bqj.apps.googleusercontent.com'
const CLIENT_ID_IOS = '53206975661-qan0rnefniegjv53ohild375pv0p7ekd.apps.googleusercontent.com'
const CLIENT_ID3 = '436789193812-5vh7ahctkaofjir9tnilfnvm19cf3vve.apps.googleusercontent.com'
const CLIENT_ID4 = '436789193812-9vubggj1op881elm41i7b9raeec9dgrj.apps.googleusercontent.com'
const CLIENT_ID5 = '436789193812-9vubggj1op881elm41i7b9raeec9dgrj.apps.googleusercontent.com'
const CLIENTS = [CLIENT_ID, CLIENT_ID2, CLIENT_ID3, CLIENT_ID4, CLIENT_ID_IOS, IOS_TEST, CLIENT_ID5]

export const verifyToken = async (req: any) => {
  let { code, token } = req.params
  code = code || token
  if (!code) {
    throw new ZError(50, 'code is required')
  }
  const client = new OAuth2Client(CLIENT_ID)
  let data: any = {}
  let openId: string
  try {
    const ticket = await client.verifyIdToken({
      idToken: code,
      audience: CLIENTS, // Specify the CLIENT_ID of the app that accesses the backend
      // Or, if multiple clients access the backend:
      //[CLIENT_ID_1, CLIENT_ID_2, CLIENT_ID_3]
    })
    const payload = ticket.getPayload()
    if (!(payload.iss === GOOGLE_OAUTH_ISS || payload.iss === GOOGLE_OAUTH_ISS1)) {
      throw new ZError(10, 'id token error')
    }
    if (CLIENTS.indexOf(payload.aud) === -1) {
      throw new ZError(11, 'client id mismatch')
    }

    if (payload.email) data.email = payload.email
    if (process.env.NODE_ENV !== 'development') {
      if (payload.email_verified !== undefined) data.emailVerified = payload.email_verified
    }
    if (payload.locale) data.locale = payload.locale
    if (payload.name) data.nickname = payload.name
    if (payload.picture) data.avatar = payload.picture
    openId = payload.sub
  } catch (err) {
    logger.log('error parse google id token', err)
    try {
      let info: any = await client.getTokenInfo(code)
      console.log(info)
      if (info.email) data.email = info.email
      if (info.aud !== CLIENT_ID2) {
        throw new ZError(11, 'client id mismatch')
      }
      if (process.env.NODE_ENV !== 'development') {
        if (info.email_verified !== undefined) data.emailVerified = info.email_verified
      }
      if (info.name) data.nickname = info.name
      openId = info.sub
    } catch (e2) {
      logger.log('error parse google access token', e2)
      throw new ZError(10, 'id token error')
    }
  }
  return { openId, data }
}
