import { INGAME_SCORE_KEY, RANK_SCORE_SCALE } from 'common/Constants'
import { InGameScoreRecord } from 'models/InGameScoreRecord'
import { InGameStats } from 'models/InGameStats'
import { ZRedisClient } from 'zutils'
const rankLevels = require('../../configs/uaw_rank_level.json')
/**
 * 更新排行榜
 * @param param0
 *          user: 用户id
 *          score: 分数
 *          scoreType: 分数类型
 *          scoreParams: 额外的参数
 */
export const updateRankScore = async ({
  user,
  score,
  scoreType,
  scoreParams,
}: {
  user: string
  score: number
  scoreType: string
  scoreParams: any
  rebateLevel?: number
}) => {
  let record = new InGameScoreRecord({
    user: user,
    score,
    type: scoreType,
    data: scoreParams,
  })
  await record.save()
  await InGameStats.updateOne({ user }, { $inc: { score } })
  const key = INGAME_SCORE_KEY
  await updateRank(key, score, user)
  // 给邀请人返利
  // if (rebateLevel > 0) {
  //   let userMod = await ActivityUser.findById(user)
  //   if (userMod?.inviteUser) {
  //     const score1 = score * INVITE_REBATE
  //     let scoreParams1 = Object.assign({ fromUser: user }, scoreParams)
  //     const keyInvite = `${activity}:invite`
  //     await updateRankInvite(keyInvite, score1, `${userMod.inviteUser}_${user}`)
  //     await updateRankScore({
  //       user: userMod.inviteUser,
  //       score: score1,
  //       activity,
  //       scoreType: SCORE_INVITE_REBATE,
  //       scoreParams: scoreParams1,
  //       rebateLevel: rebateLevel - 1,
  //     })
  //   }
  // }
}

/**
 * 更新排行榜
 * @param key
 * @param score
 * @param member
 */
export const updateRank = async (key: string, score: number, member: string) => {
  score = parseInt(score * RANK_SCORE_SCALE + '')
  await new ZRedisClient().zincrby(key, score, member)
}

export const rankLevel = (rank: number | string) => {
  rank = parseInt(rank + '')
  const data = rankLevels.find(o => rank >= o.rankMin && rank <= o.rankMax)
  return data ? data.level : '-'
}
