import { NftHolder } from 'models/chain/NftHolder'
import { ITask } from './base/ITask'
import { formatDate } from 'utils/utcdate.util'
import { NftClaimRecord } from 'models/NftClaimRecord'
import { ZError } from 'zutils'
import { InGameStats } from 'models/InGameStats'

const sourceList = require('../../configs/nft_202403.json')
const commonSet = new Set()
const legendarySet = new Set()
const rareSet = new Set()
for (let o of sourceList) {
  if (o.rarity === 'Common') {
    commonSet.add(o.id)
  } else if (o.rarity === 'Legendary') {
    legendarySet.add(o.id)
  } else if (o.rarity === 'Rare') {
    rareSet.add(o.id)
  }
}
export const tokenRarity = (tokenId: string | number) => {
  if (commonSet.has(parseInt(tokenId + ''))) {
    return 'Common'
  } else if (legendarySet.has(parseInt(tokenId + ''))) {
    return 'Legendary'
  } else if (rareSet.has(parseInt(tokenId + ''))) {
    return 'Rare'
  }
}
const rarityList = require('../../configs/nft_rarity.json')
export const rarityMap = new Map()
for (let o of rarityList) {
  rarityMap.set(o.rarity, o.ticket)
}

export default class NftHolderCheck extends ITask {
  static desc = 'nft holder'
  static show: boolean = true
  async execute(data: any) {
    return true
  }
  public async claimReward(cfg: any) {
    const { chain, address } = cfg.cfg
    let score = 0
    let ticket = 0
    const dateTag = formatDate(new Date())
    let historys = await NftClaimRecord.find({ address, dateTag })
    let historySet = new Set()
    for (let history of historys) {
      historySet.add(history.tokenId)
    }
    let records = await NftHolder.find({ chain, address, user: this.user.address.toLowerCase() })
    // remove record which tokenId in historySet first
    records = records.filter(o => !historySet.has(o.tokenId))
    if (records.length === 0) {
      throw new ZError(20, 'not NFT holder')
    }
    for (let record of records) {
      if (commonSet.has(parseInt(record.tokenId))) {
        ticket += rarityMap.get('Common')
      } else if (legendarySet.has(parseInt(record.tokenId))) {
        ticket += rarityMap.get('Legendary')
      } else if (rareSet.has(parseInt(record.tokenId))) {
        ticket += rarityMap.get('Rare')
      }
      let claimRecord = new NftClaimRecord({
        address,
        tokenId: record.tokenId,
        user: this.user.id,
        dateTag,
      })
      await claimRecord.save()
    }
    if (ticket > 0) {
      let saveRes = await InGameStats.findOneAndUpdate(
        { user: this.user.id },
        { $inc: { ticket } },
        {
          new: false,
          upsert: false,
          includeResultMetadata: true,
        },
      )
    }
    return { score, ticket }
  }

  public async check(cfg: any, gameData: any) {
    const { chain, address } = cfg.cfg
    const dateTag = formatDate(new Date())
    let historys = await NftClaimRecord.find({ address, dateTag })
    let historySet = new Set()
    for (let history of historys) {
      historySet.add(history.tokenId)
    }
    let records = await NftHolder.find({ chain, address, user: this.user.address.toLowerCase() })
    records = records.filter(o => !historySet.has(o.tokenId))
    let ticket = 0
    for (let record of records) {
      if (commonSet.has(parseInt(record.tokenId))) {
        ticket += rarityMap.get('Common')
      } else if (legendarySet.has(parseInt(record.tokenId))) {
        ticket += rarityMap.get('Legendary')
      } else if (rareSet.has(parseInt(record.tokenId))) {
        ticket += rarityMap.get('Rare')
      }
    }
    return ticket
  }
}
